@@ -5,18 +5,21 @@ except:
 
 import asyncio
 import time
-from datetime import date, datetime, timedelta
+from datetime import datetime, timedelta
 
 import aiohttp
 import numpy as np
 import pandas as pd
 import requests
 from bs4 import BeautifulSoup
+from requests.adapters import HTTPAdapter, Retry
 
 
 class Karpet:
 
     quick_search_data = None
+    req_retries = 4
+    req_backoff_factor = 3
 
     def __init__(self, start=None, end=None):
         """
@@ -28,6 +31,28 @@ class Karpet:
 
         self.start = start
         self.end = end
+        self.req_ses = self.get_session()
+
+    def get_session(self):
+
+        # Waits for 1.5s, 3s, 6s, 12s, 24s between requests.
+        status_forcelist = (500, 502, 503, 504, 429)
+
+        retry = Retry(
+            total=self.req_retries,
+            read=self.req_retries,
+            connect=self.req_retries,
+            backoff_factor=self.req_backoff_factor,
+            status_forcelist=status_forcelist,
+            method_whitelist=False,
+        )
+        adapter = HTTPAdapter(max_retries=retry)
+
+        session = requests.Session()
+        # session.mount("http://", adapter)
+        session.mount("https://", adapter)
+
+        return session
 
     def get_quick_search_data(self):
         """
@@ -53,28 +78,17 @@ class Karpet:
         :rtype: list
         """
 
-        if self.quick_search_data:
-            return self.quick_search_data
+        if not self.quick_search_data:
+            self.quick_search_data = self._get_json(
+                "https://s2.coinmarketcap.com/generated/search/quick_search.json"
+            )
 
-        url = "https://s2.coinmarketcap.com/generated/search/quick_search.json"
-
-        # Download.
-        try:
-            response = requests.get(url)
-        except:
-            raise Exception("Couldn't download necessary data from the internet.")
-
-        # Parse.
-        try:
-            self.quick_search_data = response.json()
-
-            return self.quick_search_data
-        except:
-            raise Exception("Couldn't parse downloaded data from the internet.")
+        return self.quick_search_data
 
     def fetch_crypto_historical_data(self, symbol=None, id=None):
         """
-        Retrieve basic historical information for a specific cryptocurrency from coingecko.com.
+        Retrieve basic historical information (by days) for a specific
+        cryptocurrency from coingecko.com.
         Coin ID can be retreived by get_coin_ids() method.
 
         Output dataframe has following columns:
@@ -86,7 +100,7 @@ class Karpet:
         Index is datetime64[ns].
 
         :param str symbol: Coin symbol - i.e. BTC, ETH, ...
-        :param str id: Coin ID (baed on coingecko.com).
+        :param str id: Coin ID (based on coingecko.com).
         :raises Exception: If data couldn't be download form the internet.
         :return: Dataframe with historical data.
         :rtype: pd.DataFrame
@@ -134,6 +148,48 @@ class Karpet:
 
         return df
 
+    def fetch_crypto_live_data(self, symbol=None, id=None):
+        """
+        Retrieve OHLC price data for past 24 hours for a specific
+        cryptocurrency from coingecko.com.
+        Coin ID can be retreived by get_coin_ids() method.
+
+        Output dataframe has following columns:
+
+        - open
+        - high
+        - low
+        - close
+
+        Index is datetime64[ns].
+
+        :param str symbol: Coin symbol - i.e. BTC, ETH, ...
+        :param str id: Coin ID (based on coingecko.com).
+        :raises Exception: If data couldn't be download form the internet.
+        :return: Dataframe with OHLC data.
+        :rtype: pd.DataFrame
+        """
+
+        id = self._get_coin_id_from_params(symbol, id)
+
+        # Fetch data.
+        data = []
+        url = f"https://api.coingecko.com/api/v3/coins/{id}/ohlc?vs_currency=usd&days=1"
+
+        # Fetch and check the response.
+        data = self._get_json(url)
+
+        if not data:
+            raise Exception("Couldn't download necessary data from the internet.")
+
+        # Assembly the dataframe.
+        data = np.array(data)
+        return pd.DataFrame.from_records(
+            data[:, 1:],
+            index=pd.to_datetime(data[:, 0], unit="ms"),
+            columns=["open", "high", "low", "close"],
+        )
+
     def fetch_crypto_exchanges(self, symbol=None):
         """
         Fetches all exchanges where the given symbol
@@ -203,7 +259,12 @@ class Karpet:
 
         stich_overlap = trdays - overlap
         n_days = (self.end - self.start).days
-        pytrends = TrendReq(hl=hl, tz=tz)
+        pytrends = TrendReq(
+            hl=hl,
+            tz=tz,
+            retries=self.req_retries,
+            backoff_factor=self.req_backoff_factor,
+        )
 
         # Get the dates for each search.
         if n_days <= trdays:
@@ -359,7 +420,7 @@ class Karpet:
             headers = {
                 "User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:69.0) Gecko/20100101 Firefox/69.0"
             }
-            response = requests.get("https://cointelegraph.com/", headers=headers)
+            response = self.req_ses.get("https://cointelegraph.com/", headers=headers)
             dom = BeautifulSoup(response.text, "lxml")
 
             def parse_news(news_items):
@@ -630,10 +691,12 @@ class Karpet:
 
         # Download.
         try:
-            response = requests.get(url)
+            response = self.req_ses.get(url)
         except:
             raise Exception("Couldn't download necessary data from the internet.")
 
+        response.raise_for_status()
+
         # Parse.
         try:
             return response.json()
